export(
        "sfInit",
        "sfStop",

        "sfParallel",
        "sfNodes",
        "sfCpus",
        "sfType",
        "sfIsRunning",                     ## New 1.83
        "sfGetCluster",
        "sfSession",
        "sfSocketHosts",                   ## New 1.6
        "sfSetMaxCPUs",

        "sfLibrary",
        "sfSource",
        "sfExport",
        "sfExportAll",
        "sfRemove",
        "sfRemoveAll",
        "sfCat",

        ## Snow wrappers
        "sfClusterMap",                    ## Currently not implemented.
        "sfClusterApply",
        "sfClusterApplyLB",

        ## snowfall addition: intermediate save of results (with auto-restore)
        "sfClusterApplySR",
        "sfRestore",

        "sfLapply",
        "sfSapply",
        "sfApply",
        "sfRapply",                        ## Currently not implemented.
        "sfCapply",                        ## Currently not implemented.
        "sfMM",

        "sfClusterSplit",
        "sfClusterCall",
        "sfClusterEval", "sfClusterEvalQ",
        "sfClusterSetupRNG",
        "sfClusterSetupRNGstream",
        "sfClusterSetupSPRNG",
        ## Unit tests
        "sfTest"
      )

import("snow")
importFrom("stats", "na.omit", "runif")
importFrom("utils", "assignInNamespace", "data", "getFromNamespace",
           "packageDescription")
